# -*- coding: utf_8 -*-
"""ファイルを正規表現で判別して分類して整理する
    
usage:
    python classify.py [-s][dir,dir, ...] [-j][dir,file, ...]
    
 option:
    -s[dir, ...]
       サブディレクトリも
    
    -j[dir,file, ...]
       サブディレクトリを含めない
       ファイルが指定された時はそのファイルのみ振り分ける
    
"""
# example:
#   python classify.py -s c:\download\,c:\temp\
#      c:\download\とc:\temp\内のファイルを振り分ける。サブディレクトリも。
#   
#   python classipy.py -jc:/temp/01.jpg,c:/temp/02.jpg,c:/temp/03.gif
#      指定したそれぞれのファイルを振り分ける
#
#   CraftLaunchEXに登録する場合はこんな感じ
#        <shellexec name='classify -s download directory'
#                   file='c:\bin\python\pythonw.exe'
#                   param='C:\bin\Cmdline\classify.py -s c:\download'
#                   directory='C:\bin\python'
#                   swmode=''
#        />
#
# history:
#    version 0.0.2   2005/06/24  ハッシュ比較
#    version 0.0.1   2005/06/17  初版
#
# License:
#    改造･再配布については自己責任で自由にどうぞ。
#    その際はご一報いただけると幸いです。
#
# Author:
#    シロ
#    web  SoftwareMemo http://softmemo.bblog.jp/
#    msn  sir00@hotmail.co.jp
# -------------------------------------------------------------------

import os,sys
import os.path
import re
import getopt
import time
import md5
from win32gui import MessageBox
from threading import Thread
from shutil import move

DEBUG = 0

# ログファイル
file_log      = r'C:\bin\Cmdline\classify.log'

# ルールファイル
file_rule     = r'C:\bin\Cmdline\classify.rule'

# md5が異なり、名前が同じ場合いくつまでバックアップを作るか
backup_number = 50

class MoveFile(Thread):
    def __init__(self, fr, to):
        Thread.__init__(self)
        self.fr = fr
        self.to = to
        
    def run(self):
        head,name = os.path.split(self.to)
        if not os.path.exists(head) or not os.path.isdir(head):
            os.makedirs(head)
            
        self.fr_md5 = getMd5(self.fr)
        self.to, self.to_md5 = getFileName(self.to, self.fr_md5)
        
        if DEBUG:
            return
        
        move(self.fr, self.to)
        writeLog(self.fr, self.to)
        
def getFileName(path, md5_value):
    
    if not os.path.exists(path) or getMd5(path) == md5_value:
        return path, md5_value
    
    for i in xrange(backup_number):
        cp = path + '.' + str(i) + '~'
        if not os.path.exists(cp) or getMd5(cp) == md5_value:
            break
    return cp, md5_value

def getMd5(path):
    m = md5.new()
    for f in open(path, 'rb'):
        m.update(f)
    return m.hexdigest()

def writeLog(old, new):
    try:
        f = open(file_log,'r')
        d = f.read()
    except IOERorr:
        return
    
    f.close()
    
    f = open(file_log,'w')
    data = []
    data.append(u'---------------------------------------------------------------------')
    data.append(unicode(time.strftime("%Y/%m/%d %H:%M:%S",time.localtime())))
    data.append(old.lower())
    data.append(new.lower() + u'\n\n')
    f.write(u'\n'.join(data))
    f.write(d)
    f.close()
    
def getRule(path):
    for line in open(path, 'r'):
        if not re.match(r'^\s*(#|\n)', line):
            res = unicode(line, 'mbcs').rstrip()
            yield tuple(re.split('\t+', res))
            
def usage():
    MessageBox(0, unicode(__doc__, 'utf_8'), 'classify.py', 0)

def getFiles(args):
    for path in args:
        if os.path.isdir(unicode(path)):
            for name in os.listdir(path):
                yield name, os.path.join(path, name)
        else:
            yield  os.path.split(path)[1].lower(), path.lower()
            
def myGlob(path):
    for p in path:
        for root, dirs, files in os.walk(p):
            for name in files:
                yield name.lower(), os.path.join(root,name).lower()
                
def main(argv):
    try:
        optlist, args = getopt.getopt(argv, "s:j:")
    except getopt.GetoptError, e:
        print 'e'
        usage()
        sys.exit(1)
        
    opt = dict(optlist)
    
    if opt.has_key('-s'):
        # remove file path, encode, normpath
        s = [os.path.normpath(unicode(x)) for x in opt['-s'].split(',') if os.path.isdir(x) ]
        
        for name, path_old in myGlob(s):
            for rule, path_new in getRule(file_rule):
                if re.search(rule, path_old):
                    MoveFile(path_old, os.path.join(path_new, name)).start()
                    
    if opt.has_key('-j'):
        # encode, normpath
        j = [os.path.normpath(unicode(x)) for x in opt['-j'].split(',')]
        
        for name, path_old in getFiles(j):
            for rule, path_new in getRule(file_rule):
                if re.search(rule, path_old):
                    MoveFile(path_old, os.path.join(path_new, name)).start()
                    
if __name__ == "__main__":
    if len(sys.argv) == 1:
        usage()
    else:
        main(sys.argv[1:])
